// --- VESCTelemetry_V1.60.ino --- //
// Designed to work on Arduino Nano (2KB sram  32KB flash 1KB EEPROM)
// https://github.com/SolidGeek/VescUart and https://github.com/cbm80amiga/Arduino_ST7789_Fast libraries
// Vibe coded by David Bloomfield using Gemini 2.5 Pro Exp, inspired by Lukas Janky's work
// Gemini 2nd Mod (2025-04-21): Fix layout Screen 4 (Dist label, Avg Cons units), Fix Settings highlight.
// Gemini 3rd Mod (2025-04-21): Implement conditional EEPROM save logic, adjust Screen 4 layout further.
// Gemini 4th Mod (2025-04-21): Correct distance calc factor (3.0->6.0), adjust Screen 4 unit layout again.
// Gemini 5th Mod (2025-04-21): Add explicit clear rect for speed display on Screen 1 to fix artifacts.
// Gemini 6th Mod (2025-04-22): Optimize Screen 1 speed display to redraw only on value change, reducing flicker.
// Gemini 7th Mod (2025-04-22): Add regen color indication, reorder Screen 2, apply conditional draw to more elements.
// Gemini 8th Mod (2025-04-22): Fix compilation errors due to incorrect color define names.
// Gemini 9th Mod (2025-04-22): Revert extensive conditional drawing to fix RAM-related boot loop. Keep speed optimization.
// Gemini 10th Mod(2025-04-22): Increase screenUpdateInterval to 100ms (10Hz) to reduce flicker and improve button responsiveness.
// Gemini 11th Mod(2025-04-22): Decrease screenUpdateInterval to 33ms (~30Hz) by user request.
// Changes: Rewrote Screen 3 drawing without helper, drawing all text with BG color
//          to try and fix label flicker specific to that screen.
//          Includes previous fixes.
// Added: Global constants, Odometer (EEPROM), Screen 4 redesign, Settings update, Pole pair fix, Power display fix.
// V1.40 (2025-04-27): Reworked Screen Sleep and EEPROM Odo Save logic, Added Software Version.
// V1.41 (2025-04-30): Use direct UART.data.wattHours, Draw Screen 1 Speed with explicit background.
// V1.42 (2025-04-30): Use F() macro for string literals to save RAM.
// V1.43 (2025-04-30): Calculate NET Wh Used (Discharged - Charged).
// V1.44 (2025-04-30): Calculate NET Ah Used (Discharged - Charged).
// V1.45 (2025-04-30): Add Lifetime Average Efficiency (Avg Eff), rename Trip Avg Cons to Trip Eff.
// V1.46 (2025-04-30): Reduce VESC UART polling frequency to ~10Hz.
// V1.47 (2025-04-30): Decouple screen updates from VESC polling.
// V1.48 (2025-04-30): Add button debouncing logic.
// V1.49 (2025-04-30): Further reduce VESC UART polling frequency to ~5Hz.
// V1.50 (2025-04-30): Revert button debouncing logic.
// V1.51 (2025-04-30): Restore Regen Color + Add for Wh/Ah Used on Screen 2 & 4.
// V1.52 (2025-04-30): Calculate Avg Eff using NET lifetime Wh.
// V1.53 (2025-04-30): Correctly save/load/update Lifetime NET Wh for Avg Eff.
// V1.54 (2025-05-01): Restructure Screen 2 (Motor) and Reorder Screen 4 (Trip Info).
// V1.55 (2025-05-01): Fix Avg Eff reset on restart by correctly accumulating lifetimeNetWattHour.
// V1.56 (2025-05-01): Fix potential crash by using stored prevVescWattHours in CLEAR ODO logic.
// V1.58 (2025-05-01): Removed unused global variables (maxa, maxvel, whkm) to save SRAM, keeping V1.56 Avg Eff logic.
// V1.59 (2025-05-01): Adjusted Screen 1 layout: Increased velocity clear height, shifted battery % left slightly.
// V1.60 (2025-05-02): Increased font size on Screen 2 to Size 3, adjusted layout and labels to fit.

#include <VescUart.h>
#include <EEPROM.h> // Include EEPROM library for persistence

//Library for the Display
#include <SPI.h>
#include <Adafruit_GFX.h>
#include <Arduino_ST7789_Fast.h> // Using the Fast library is good!
//--------------------------------------------------------------------------------------------------
// --- GLOBAL CONFIGURATION CONSTANTS (ADJUST THESE FOR YOUR SETUP) ---
//--------------------------------------------------------------------------------------------------
const char* SOFTWARE_VERSION = "V1.60"; // <<< Software version string

// --- System / Hardware ---
const int POLE_PAIRS = 3; // <<< Number of motor pole pairs (physical RPM = eRPM / POLE_PAIRS)
const float WHEEL_DIAMETER_M = 0.2794; // Wheel diameter in meters (e.g., 11 inches * 0.0254 m/inch)
const float GEAR_RATIO = 7.056; // Motor pulley teeth / Wheel pulley teeth (or 1.0 if direct drive)
const int BUTTON_PIN = 2; // Pin for the button

// --- Battery ---
const int CELL_COUNT = 10; // Number of cells in series in your battery pack
const float MIN_VOLTAGE = 34.0; // Voltage at 0% battery (e.g., 3.4V * CELL_COUNT)
const float MAX_VOLTAGE = 42.0; // Voltage at 100% battery (e.g., 4.2V * CELL_COUNT)

// --- Odometer ---
const float ODO_SAVE_SPEED_THRESHOLD_KMH = 2.0; // Speed threshold (in km/h) below which saving is allowed
const float ODO_SAVE_DISTANCE_DIFF_KM = 0.1; // Min distance travelled (in km) since last save to trigger a new save [NEW]

// --- Screen Sleep --- [NEW]
const unsigned long SCREEN_SLEEP_DELAY_MS = 120000UL; // 2 minutes in milliseconds [NEW]
const float SCREEN_SLEEP_SPEED_THRESHOLD = 0.0; // Speed threshold for initiating sleep timer [NEW]
const float SCREEN_WAKE_SPEED_THRESHOLD = 0.0; // Speed threshold for waking screen [NEW]

// --- Button Handling ---
const unsigned long LONG_PRESS_DURATION = 1000; // ms
// Debounce delay removed

// --- EEPROM Addresses ---
const int EEPROM_ADDR_UNITS = 0; // 1 byte (0 or 1)
const int EEPROM_ADDR_ROTATION = 1;    // 1 byte (0-3)
const int EEPROM_ADDR_ODOMETER = 2; // Start address for odometer float (4 bytes)
const int EEPROM_ADDR_LIFETIME_NET_WH = 6; // Start address for lifetime NET Wh float (4 bytes) - RENAMED
const int EEPROM_SIZE = 16; // Minimum EEPROM size needed (now needs at least 10)

//--------------------------------------------------------------------------------------------------
// --- END OF GLOBAL CONFIGURATION CONSTANTS ---
//--------------------------------------------------------------------------------------------------

//#define TFT_CS    6 // Keep commented if using hardware CS
#define TFT_DC 9
#define TFT_RST 8
#define SCR_WD 240
#define SCR_HT 240
Arduino_ST7789 lcd = Arduino_ST7789(TFT_DC, TFT_RST); //
//Arduino_ST7789 lcd = Arduino_ST7789(TFT_DC, TFT_RST, TFT_CS); // Uncomment if using software CS on pin 6

// Colors
#define Black 0x0000       /* 0,   0,   0 */
#define Navy 0x000F        /* 0,   0, 128 */
#define DarkGreen 0x03E0   /* 0, 128,   0 */
#define DarkCyan 0x03EF    /* 0, 128, 128 */
#define Maroon 0x7800      /* 128,   0,   0 */
#define Purple 0x780F      /* 128,   0, 128 */
#define Olive 0x7BE0       /* 128, 128,   0 */
#define LightGrey 0xD69A   /* 211, 211, 211 */
#define DarkGrey 0x7BEF    /* 128, 128, 128 */
#define Blue 0x001F        /* 0,   0, 255 */
#define Green 0x07E0       /* 0, 255,   0 */ // Used for Regen indication
#define Cyan 0x07FF        /* 0, 255, 255 */
#define Red 0xF800         /* 255,   0,   0 */ // Default Label/Unit color
#define Magenta 0xF81F     /* 255,   0, 255 */
#define Yellow 0xFFE0      /* 255, 255,   0 */
#define White 0xFFFF       /* 255, 255, 255 */ // Default Value color
#define Orange 0xFD20      /* 255, 165,   0 */
#define GreenYellow 0xAFE5 /* 173, 255,  47 */
#define Pink 0xF81F

// Define custom colors
#define DEFAULT_LABEL_COLOR Red
#define DEFAULT_VALUE_COLOR White
#define DEFAULT_UNIT_COLOR Red
#define REGEN_COLOR Green // Color for labels/units during regen
#define SETTINGS_ITEM_COLOR White
#define SETTINGS_FOCUS_COLOR White
#define SETTINGS_FOCUS_BG_COLOR DarkGrey
#define SETTINGS_SELECTED_COLOR White
#define SEPARATOR_DOT_COLOR DarkGrey
#define TITLE_SEPARATOR_COLOR White

/** Initiate VescUart class */
VescUart UART;

// VESC Data Variables (Raw Metric) - These hold the latest successful read
int rpm = 0;
float voltage = 0.0;
float current = 0.0; // Battery current
float phase_current = 0.0; // Phase current
int power = 0;
float amphour = 0.0; // Raw NET Ah (Calculated: Discharged - Charged) for current trip
float tach = 0.0;
float distance = 0.0; // Current trip distance (Calculated from tachometer)
float velocity = 0.0; // INSTANTANEOUS velocity in km/h (Calculated from RPM)
float watthour = 0.0; // Raw NET Wh (Calculated: Discharged - Charged) for CURRENT TRIP ONLY
float batpercentage = 0.0;
float temp_fet = 0.0;
float temp_motor = 0.0;

// Odometer & Lifetime Stats Variables
float odometer = 0.0; // Lifetime distance
float lifetimeNetWattHour = 0.0; // Lifetime NET Wh (Discharged - Charged) - Loaded from EEPROM, ACCUMULATED by VESC data
float lastDistanceOdo = 0.0; // Tracks distance for ODO increment
float odoLastSavedValue = 0.0; // Tracks ODO value at last EEPROM save
float lifetimeNetWhLastSavedValue = 0.0; // Tracks lifetime NET Wh at last EEPROM save - RENAMED

// Variables to track previous VESC Wh values for lifetime accumulation
float prevVescWattHours = 0.0f;
float prevVescWattHoursCharged = 0.0f;
bool vescWhInitialized = false; // Flag to ensure initial values are set correctly

// Display Variables (potentially converted) - Updated in updateDisplayValues()
float displayVelocity = 0.0;
float displayDistance = 0.0;
float displayOdometer = 0.0;
float displayWhUsed = 0.0; // Display value for NET Watt-hours used (current trip)
float displayAhUsed = 0.0; // Display value for NET Amp-hours used (current trip)
float displayTripEff = 0.0; // Display value for Trip Efficiency (Wh/dist) - RENAMED
float displayAvgEff = 0.0;  // Display value for Lifetime Average Efficiency (Wh/dist) - NEW
float displayTempFet = 0.0;
float displayTempMotor = 0.0;
float displayMaxFetTemp = 0.0;
float displayMaxMotorTemp = 0.0;
float displayBatPercentage = 0.0; // Separate display variable for battery %
float displayCellVoltage = 0.0; // Separate display variable for cell voltage
int   displayPower = 0; // Separate display variable for power
// Use char arrays for units as F() macro doesn't work directly with strcpy
char velocityUnitStr[5]; // Initialized in updateDisplayValues
char distanceUnitStr[4]; // Initialized in updateDisplayValues
char odoUnitStr[4];      // Initialized in updateDisplayValues
char tempUnitStr[2];     // Initialized in updateDisplayValues
char consumptionUnitStr[6]; // Initialized in updateDisplayValues (Used for both Trip and Avg Eff)
char energyUnitStr[4];    // Initialized in updateDisplayValues
char capacityUnitStr[4];  // Unit for Ah Used ("Ah")

// --- State variable ONLY for conditional speed drawing ---
const float UNINITIALIZED_FLOAT = -999.9f;
float lastDispVelocity1 = UNINITIALIZED_FLOAT; // Track only speed for Screen 1 optimization

// State variables
#define DISPLAY_STATE_MAX 4
#define SETTINGS_STATE 0
int currentState = 1;
int lastState = -1;
unsigned long lastButtonPressTime = 0; // Used for long press detection

// Settings state variables
#define NUM_SETTINGS_ITEMS 9
#define RESERVED_FOCUS_INDEX 6
#define CLEAR_ODO_FOCUS_INDEX 7 // Clears ODO and Avg Eff stats
#define EXIT_FOCUS_INDEX 8
int settingsFocus = 0;
int currentUnits = 0;
int currentRotation = 2;

// Button State Variables (Simple Press/Release) - REVERTED
bool buttonIsDown = false; // Tracks if button is currently pressed
unsigned long buttonDownTime = 0; // Time when button was pressed
bool longPressActive = false; // Tracks if a long press is currently active

// Screen Timeout Variables [MODIFIED]
unsigned long screenSleepTimerStart = 0; // Time when speed dropped below threshold [NEW]
bool isScreenOff = false;
bool screenSleepTimerActive = false; // Flag if the sleep timer is running [NEW]

// Timing & Throttling Variables
unsigned long lastScreenUpdateTime = 0;
const unsigned long screenUpdateInterval = 33; // Target ~30Hz screen update rate
unsigned long lastVescPollTime = 0; // Time of last successful VESC poll - NEW
const unsigned long vescUpdateInterval = 200; // Poll VESC at ~5Hz (was 100ms) - MODIFIED

// Maximum values (Raw Metric) - Updated when VESC data is read
float maxfetemp = -999.0f; // Keep max temps as they are displayed on Screen 3
float maxmotortemp = -999.0f;

// Conversion Factors
const float KM_TO_MILES = 0.621371;
const float MILES_TO_KM = 1.60934;

// Alignment constants
const int labelStartX1 = 15;
const int valueStartX1 = 145;
const int unitStartX1 = 205;
const int velocity_y = 45;
const int velocity_height = 85; // Increased from 75 to prevent digit artifacts
const int labelStartX2 = 10;  // Adjusted for Screen 2 Size 3 Font
const int valueStartX2 = 115; // Adjusted for Screen 2 Size 3 Font
const int unitStartX2 = 210;  // Adjusted for Screen 2 Size 3 Font
const int labelStartX3 = 15;
const int valueStartX3 = 150;
const int settingsLabelX = 15;
const int settingsOptionIndent = 30;
const int settingsUnitsOptionX = settingsOptionIndent;
const int settingsUnitsOptionSpacing = 100;
const int settingsRotOptionX = settingsOptionIndent;
const int settingsRotOptionSpacing = 50;
const int rowSeparatorMargin = 5; // Margin between text and separator line
const int separatorDotSpacing = 4;
const int separatorStartX = 25;
const int separatorEndX = SCR_WD - separatorStartX;
const int titleSeparatorHeight = 3;

// Function Prototypes
void drawScreen1();
void drawScreen2();
void drawScreen3();
void drawScreen4();
void drawSettingsScreen();
void drawDottedLine(int x0, int y, int x1, uint16_t color, int spacing);
void updateDisplayValues(); // Updates display variables from raw variables
void processVescData();   // Processes data after a successful VESC read
void saveLifetimeStats(); // Modified to save Odo and Lifetime NET Wh
// void handleButton(); // Removed debounced handler

//SETUP...............................................................................................................................................................
void setup() {
  Serial.begin(115200);
  lcd.init(SCR_WD, SCR_HT);

  // --- Load Settings & Lifetime Stats ---
  // EEPROM.begin(EEPROM_SIZE); // Uncomment if needed

  byte storedUnits = EEPROM.read(EEPROM_ADDR_UNITS);
  byte storedRotation = EEPROM.read(EEPROM_ADDR_ROTATION);
  EEPROM.get(EEPROM_ADDR_ODOMETER, odometer);
  EEPROM.get(EEPROM_ADDR_LIFETIME_NET_WH, lifetimeNetWattHour); // Load lifetime NET Wh - RENAMED

  // Validate Odometer
  if (isnan(odometer) || odometer < 0.0f || odometer > 999999.0f) {
      odometer = 0.0f;
      EEPROM.put(EEPROM_ADDR_ODOMETER, odometer);
  }
  // Validate Lifetime Net Wh - MODIFIED
  if (isnan(lifetimeNetWattHour) || lifetimeNetWattHour < -999999.0f || lifetimeNetWattHour > 9999999.0f) { // Allow negative range
      lifetimeNetWattHour = 0.0f;
      EEPROM.put(EEPROM_ADDR_LIFETIME_NET_WH, lifetimeNetWattHour);
  }

  odoLastSavedValue = odometer; // Initialize last saved value
  lifetimeNetWhLastSavedValue = lifetimeNetWattHour; // Initialize last saved lifetime NET Wh value - RENAMED
  lastDistanceOdo = 0.0;

  currentUnits = (storedUnits > 1) ? 0 : storedUnits;
  if (storedUnits > 1) EEPROM.write(EEPROM_ADDR_UNITS, currentUnits);

  currentRotation = (storedRotation > 3) ? 2 : storedRotation;
  if (storedRotation > 3) EEPROM.write(EEPROM_ADDR_ROTATION, currentRotation);

  // EEPROM.commit(); // Uncomment if needed

  lcd.setRotation(currentRotation);

  // --- Initial Display ---
  lcd.fillScreen(BLACK);
  lcd.setCursor(10, 100); lcd.setTextColor(WHITE); lcd.setTextSize(3);
  lcd.println(F("INITIALIZING")); // Use F() macro
  lcd.setCursor(85, 130); lcd.print(F("VESC")); // Use F() macro

  // --- ADDED: Display Software Version ---
  lcd.setTextSize(2); // Adjust size if needed
  lcd.setCursor(80, 155); // Adjust position (X, Y) as needed
  lcd.print(SOFTWARE_VERSION); // Version string is already const char*
  // --- END OF ADDED CODE ---

  UART.setSerialPort(&Serial);
  pinMode(BUTTON_PIN, INPUT_PULLUP); // Use constant for button pin

  delay(1500);

  // Initialize state variables
  lastState = -1;
  currentState = 1;
  isScreenOff = false;
  screenSleepTimerStart = 0; // [NEW]
  screenSleepTimerActive = false; // [NEW]
  lastScreenUpdateTime = 0; // Initialize timers
  lastVescPollTime = 0;     // Initialize timers
  lastDispVelocity1 = UNINITIALIZED_FLOAT; // Initialize only speed tracking
  vescWhInitialized = false; // Mark Wh values as not yet initialized

  lcd.fillScreen(BLACK);
  // Try an initial VESC read before first draw
  if (UART.getVescValues()) {
      lastVescPollTime = millis(); // Update poll time
      // Initialize prev Wh values AFTER first successful read
      prevVescWattHours = UART.data.wattHours;
      prevVescWattHoursCharged = UART.data.wattHoursCharged;
      vescWhInitialized = true; // Mark as initialized
      processVescData(); // Process the initial data (will use initialized prev values)
  } else {
      // Set default values if initial read fails
      batpercentage = 0.0f; velocity = 0.0f; distance = 0.0f;
      // prevVescWattHours/Charged remain 0.0f, vescWhInitialized remains false
      // Use loaded lifetime values even if initial read fails
  }

  updateDisplayValues(); // Update display variables based on initial read or defaults
  drawScreen1(); // Draw screen 1
  lastState = currentState;
  lastScreenUpdateTime = millis(); // Set screen update time after drawing
}

// Processes the data received from VESC - MODIFIED for Lifetime Wh Accumulation
void processVescData() {
    // --- Assign RAW metric values ---
    rpm = (UART.data.rpm) / POLE_PAIRS;
    voltage = (UART.data.inpVoltage);
    current = (UART.data.avgInputCurrent); // Batt A
    phase_current = (UART.data.avgMotorCurrent); // Phase A
    power = voltage * current; // Power W (Instantaneous)
    temp_fet = (UART.data.tempMosfet);
    temp_motor = (UART.data.tempMotor);
    // Calculate NET Amp-hours (Discharged - Charged) for current trip
    amphour = UART.data.ampHours - UART.data.ampHoursCharged;
    // Calculate NET Watt-hours (Discharged - Charged) for CURRENT TRIP ONLY
    watthour = UART.data.wattHours - UART.data.wattHoursCharged;
    tach = (UART.data.tachometerAbs);

    // --- Accumulate Lifetime NET Wh ---
    if (vescWhInitialized) {
        // Calculate the CHANGE in NET Watt-hours since the last successful read
        float deltaNetWh = (UART.data.wattHours - prevVescWattHours) - (UART.data.wattHoursCharged - prevVescWattHoursCharged);
        // Accumulate the change into the LIFETIME total
        lifetimeNetWattHour += deltaNetWh;
    } else {
        // If VESC Wh values weren't initialized in setup (e.g., first read failed),
        // initialize them now with the current values. Don't calculate delta yet.
        vescWhInitialized = true;
    }
    // Update the previous values for the next calculation *after* using them
    prevVescWattHours = UART.data.wattHours;
    prevVescWattHoursCharged = UART.data.wattHoursCharged;
    // --- END OF Lifetime Wh Accumulation ---


    // --- Calculate RAW Metric derived values ---
    const float M_PER_KM = 1000.0;
    const float SEC_PER_MIN = 60.0;
    const float PULSES_PER_MOTOR_REV = POLE_PAIRS * 6.0; // Corrected factor

    float motor_revolutions = (PULSES_PER_MOTOR_REV > 0) ? UART.data.tachometerAbs / PULSES_PER_MOTOR_REV : 0;
    velocity = (rpm / GEAR_RATIO) * PI * WHEEL_DIAMETER_M * (SEC_PER_MIN / M_PER_KM);
    // Calculate distance based on change in tachometer, NOT motor_revolutions directly
    distance = (motor_revolutions / GEAR_RATIO) * PI * WHEEL_DIAMETER_M / M_PER_KM; // Review this calculation if trip reset is needed

    const float VOLTAGE_RANGE = MAX_VOLTAGE - MIN_VOLTAGE;
    float clamped_voltage = constrain(voltage, MIN_VOLTAGE, MAX_VOLTAGE);
    if (VOLTAGE_RANGE > 0) { batpercentage = ((clamped_voltage - MIN_VOLTAGE) / VOLTAGE_RANGE) * 100.0; }
    else { batpercentage = (voltage >= MAX_VOLTAGE) ? 100.0 : 0.0; }
    batpercentage = constrain(batpercentage, 0.0, 100.0);

    // --- Update Maximums ---
    if (temp_fet > maxfetemp) { maxfetemp = temp_fet; }
    if (temp_motor > maxmotortemp) { maxmotortemp = temp_motor; }

    // --- Update Odometer ---
    if (distance > lastDistanceOdo && distance < (lastDistanceOdo + 100.0)) { // Basic sanity check added
        float distanceIncrement = distance - lastDistanceOdo;
        odometer += distanceIncrement;
    }
    lastDistanceOdo = distance; // Update lastDistanceOdo only after successful read

    // --- Check Lifetime Stats Save Condition ---
    // Save if speed is low AND enough distance OR net Wh has changed significantly
    // Uses the correctly accumulated lifetimeNetWattHour now
    bool shouldSave = velocity < ODO_SAVE_SPEED_THRESHOLD_KMH &&
                      ( (odometer - odoLastSavedValue) > ODO_SAVE_DISTANCE_DIFF_KM ||
                        abs(lifetimeNetWattHour - lifetimeNetWhLastSavedValue) > 1.0 );
    if (shouldSave) {
        saveLifetimeStats();
    }
}


// Function to handle saving Odometer, Lifetime NET Wh and updating last saved values - MODIFIED
void saveLifetimeStats() {
    // Serial.print(F("Saving ODO: ")); Serial.print(odometer);
    // Serial.print(F(" | Saving NetWh: ")); Serial.println(lifetimeNetWattHour); // Now saves accumulated value
    EEPROM.put(EEPROM_ADDR_ODOMETER, odometer);
    EEPROM.put(EEPROM_ADDR_LIFETIME_NET_WH, lifetimeNetWattHour); // Save the current ACCUMULATED LIFETIME NET Wh value
    // EEPROM.commit(); // Uncomment if needed
    odoLastSavedValue = odometer; // Update the last saved value
    lifetimeNetWhLastSavedValue = lifetimeNetWattHour; // Update the last saved tracking variable
}

// Removed handleButton function

//LOOP START............................................................................................................................................................
void loop() {
  bool vescPolledSuccessfully = false; // Flag for this loop iteration
  bool shortPressTrigger = false;      // Flag for short press action
  bool longPressTrigger = false;       // Flag for long press action
  unsigned long currentMillis = millis();

  ////////// Read values from VESC (Throttled) //////////
  if (currentMillis - lastVescPollTime >= vescUpdateInterval) {
      if (UART.getVescValues()) {
          vescPolledSuccessfully = true; // Set flag ONLY on successful read
          lastVescPollTime = currentMillis; // Update time of last successful poll
          processVescData(); // Process the newly received data (updates lifetime Wh correctly now)
      } // End if (UART.getVescValues())
      else {
          lastVescPollTime = currentMillis; // Update time even on failure to prevent rapid retries
          // Optional: Handle failed VESC read if needed
          // Serial.println(F("VESC read failed!"));
          // If read fails, prevVescWattHours/Charged retain their last known values.
          // The delta calculation will resume correctly on the next successful read.
      }
  } // End if (currentMillis - lastVescPollTime >= vescUpdateInterval)


  ////////BUTTON LOGIC (Simple Press/Release)//////// - REVERTED
  if (digitalRead(BUTTON_PIN) == LOW) { // Button is pressed
      if (!buttonIsDown) { // If it wasn't pressed before
          buttonIsDown = true;
          buttonDownTime = currentMillis; // Record the time it was pressed
          longPressActive = false; // Reset long press flag
      } else { // Button is still being held down
          // Check if long press duration has been reached
          if (!longPressActive && (currentMillis - buttonDownTime >= LONG_PRESS_DURATION)) {
              longPressTrigger = true; // Set long press trigger flag
              longPressActive = true; // Mark long press as active (prevents short press on release)
          }
      }
  } else { // Button is not pressed (HIGH)
      if (buttonIsDown) { // If it *was* pressed before (i.e., it was just released)
          buttonIsDown = false; // Mark button as released
          if (!longPressActive) { // Only trigger short press if long press wasn't active
              shortPressTrigger = true; // Set short press trigger flag
          }
          // Reset long press active state regardless (it's handled by the trigger flag now)
          longPressActive = false;
      }
  }


  // --- Process Button Actions Based on Trigger Flags ---
  bool forceFullRedraw = false; // Flag to force a redraw regardless of timers/data change
  bool settingsChanged = false; // Flag if a setting was changed

  if (currentState == SETTINGS_STATE) {
      // Settings Menu Actions
      if (shortPressTrigger) {
          settingsFocus = (settingsFocus + 1) % NUM_SETTINGS_ITEMS;
          if (settingsFocus == RESERVED_FOCUS_INDEX) settingsFocus = (settingsFocus + 1) % NUM_SETTINGS_ITEMS; // Skip reserved
          forceFullRedraw = true; // Force redraw on focus change
      } else if (longPressTrigger) {
          switch (settingsFocus) {
              case 0: if (currentUnits != 1) { currentUnits = 1; settingsChanged = true; } break;
              case 1: if (currentUnits != 0) { currentUnits = 0; settingsChanged = true; } break;
              case 2: if (currentRotation != 0) { currentRotation = 0; settingsChanged = true; lcd.setRotation(currentRotation); } break;
              case 3: if (currentRotation != 1) { currentRotation = 1; settingsChanged = true; lcd.setRotation(currentRotation); } break;
              case 4: if (currentRotation != 2) { currentRotation = 2; settingsChanged = true; lcd.setRotation(currentRotation); } break;
              case 5: if (currentRotation != 3) { currentRotation = 3; settingsChanged = true; lcd.setRotation(currentRotation); } break;
              case CLEAR_ODO_FOCUS_INDEX: // Modified to clear ODO and Lifetime NET Wh
                  odometer = 0.0f;
                  lifetimeNetWattHour = 0.0f; // Clear lifetime NET Wh
                  lastDistanceOdo = distance; // Reset tracking distance based on last known distance
                  odoLastSavedValue = odometer;
                  lifetimeNetWhLastSavedValue = lifetimeNetWattHour; // Update tracking variable

                  // Reset previous VESC Wh values using the STORED previous values
                  // (No change needed here from V1.56 logic)

                  saveLifetimeStats(); // Save the cleared values immediately
                  settingsChanged = true; break;
              case EXIT_FOCUS_INDEX:
                  currentState = 1;
                  // Save Lifetime Stats on exit if conditions met (safety check)
                   bool shouldSaveExit = velocity < ODO_SAVE_SPEED_THRESHOLD_KMH &&
                                        ( (odometer - odoLastSavedValue) > ODO_SAVE_DISTANCE_DIFF_KM ||
                                          abs(lifetimeNetWattHour - lifetimeNetWhLastSavedValue) > 1.0 );
                  if (shouldSaveExit) {
                      saveLifetimeStats();
                  }
                  settingsChanged = true; // Mark setting changed to trigger updateDisplayValues
                  forceFullRedraw = true; // Force redraw when exiting settings
                  break;
          }
          if (settingsChanged) {
              EEPROM.write(EEPROM_ADDR_UNITS, currentUnits);
              EEPROM.write(EEPROM_ADDR_ROTATION, currentRotation);
              // EEPROM.commit();
              forceFullRedraw = true; // Force redraw after setting change
          }
      }
  } else {
      // Normal Display Screen Actions
      if (shortPressTrigger) {
          currentState = (currentState % DISPLAY_STATE_MAX) + 1; // Cycle 1->2->3->4->1
          forceFullRedraw = true;
      } else if (longPressTrigger) {
          // Save Lifetime Stats before entering settings if conditions met (safety check)
          bool shouldSaveEnterSet = velocity < ODO_SAVE_SPEED_THRESHOLD_KMH &&
                                   ( (odometer - odoLastSavedValue) > ODO_SAVE_DISTANCE_DIFF_KM ||
                                     abs(lifetimeNetWattHour - lifetimeNetWhLastSavedValue) > 1.0 );
          if (shouldSaveEnterSet) {
             saveLifetimeStats();
          }
          currentState = SETTINGS_STATE;
          settingsFocus = 0; forceFullRedraw = true;
      }
  }

  // --- Screen Sleep/Wake Logic ---
  bool screenJustWoke = false;
  float currentVelocityForCheck = velocity; // Use latest velocity if available

  // Wake condition: Button press OR Speed > Wake Threshold
  // Use trigger flags for button wake now
  if (isScreenOff && (shortPressTrigger || longPressTrigger || (currentVelocityForCheck > SCREEN_WAKE_SPEED_THRESHOLD))) {
      isScreenOff = false;
      screenJustWoke = true;
      forceFullRedraw = true;
      screenSleepTimerActive = false; // Stop timer on wake
      // Serial.println(F("Screen WAKE")); // Debug with F()
  }

  // Sleep condition check (only if screen is ON)
  if (!isScreenOff) {
      if (currentVelocityForCheck <= SCREEN_SLEEP_SPEED_THRESHOLD) {
          // Speed is low, start or continue timer
          if (!screenSleepTimerActive) {
              screenSleepTimerActive = true;
              screenSleepTimerStart = currentMillis;
              // Serial.println(F("Screen sleep timer STARTED")); // Debug with F()
          } else {
              // Timer is active, check if delay has elapsed
              if (currentMillis - screenSleepTimerStart >= SCREEN_SLEEP_DELAY_MS) {
                  isScreenOff = true;
                  screenSleepTimerActive = false; // Stop timer
                  // Save Lifetime Stats before sleep if conditions met (safety check)
                   bool shouldSaveSleep = currentVelocityForCheck < ODO_SAVE_SPEED_THRESHOLD_KMH &&
                                         ( (odometer - odoLastSavedValue) > ODO_SAVE_DISTANCE_DIFF_KM ||
                                           abs(lifetimeNetWattHour - lifetimeNetWhLastSavedValue) > 1.0 );
                  if (shouldSaveSleep) {
                     saveLifetimeStats();
                  }
                  lcd.fillScreen(BLACK); // Turn off screen
                  // Serial.println(F("Screen SLEEP")); // Debug with F()
                  return; // Skip the rest of the loop if screen is now off
              }
          }
      } else { // Speed is high enough
          // Speed is high, stop the sleep timer if it was active
          if (screenSleepTimerActive) {
              screenSleepTimerActive = false;
              // Serial.println(F("Screen sleep timer STOPPED (speed up)")); // Debug with F()
          }
      }
  }

  // --- DISPLAY UPDATE LOGIC ---
  bool stateChanged = (currentState != lastState); // State change always forces redraw
  bool fullRedrawNeeded = stateChanged || screenJustWoke || forceFullRedraw;
  bool screenIntervalElapsed = (currentMillis - lastScreenUpdateTime >= screenUpdateInterval);

  // Update display variables if settings changed OR new VESC data arrived
  if (settingsChanged || vescPolledSuccessfully) {
      updateDisplayValues();
  }

  // Determine if redraw is needed: Screen ON AND (forced OR interval elapsed)
  bool needsRedraw = !isScreenOff && (fullRedrawNeeded || screenIntervalElapsed);

  if (needsRedraw) {
      if (fullRedrawNeeded) {
          lcd.fillScreen(BLACK);
          lastDispVelocity1 = UNINITIALIZED_FLOAT; // Reset speed tracking on full redraw
      }

      // Draw the current screen (settings or data)
      // Drawing functions use the latest values in 'display*' variables
      switch (currentState) {
          case SETTINGS_STATE: drawSettingsScreen(); break;
          case 1: drawScreen1(); break;
          case 2: drawScreen2(); break;
          case 3: drawScreen3(); break;
          case 4: drawScreen4(); break;
      }

      lastState = currentState; // Update lastState *after* drawing
      lastScreenUpdateTime = currentMillis; // Update last screen time *after* drawing
  }
} // END OF loop()


// --- Unit Conversion Function ---
// Calculates and converts Trip Eff and Avg Eff
// Reads from global VESC data variables, writes to global 'display*' variables
void updateDisplayValues() {
    float odoForCalc; // Odometer value in the correct unit for efficiency calculation
    float distForCalc; // Trip distance value in the correct unit

    if (currentUnits == 1) { // Standard Units (Miles)
        displayVelocity = velocity * KM_TO_MILES;
        displayDistance = distance * KM_TO_MILES;
        displayOdometer = odometer * KM_TO_MILES;
        odoForCalc = displayOdometer; // Use miles for calculation
        distForCalc = displayDistance; // Use miles for calculation
        displayWhUsed = watthour; // Wh is Wh, no conversion needed (using CURRENT TRIP net Wh)
        displayAhUsed = amphour;  // Ah is Ah, no conversion needed (using CURRENT TRIP net Ah)
        displayTempFet = (temp_fet * 9.0 / 5.0) + 32.0;
        displayTempMotor = (temp_motor * 9.0 / 5.0) + 32.0;
        displayMaxFetTemp = (maxfetemp * 9.0 / 5.0) + 32.0;
        displayMaxMotorTemp = (maxmotortemp * 9.0 / 5.0) + 32.0;
        displayBatPercentage = batpercentage;
        displayCellVoltage = (CELL_COUNT > 0) ? voltage / (float)CELL_COUNT : 0.0;
        displayPower = power;


        // Assign unit strings
        strcpy(velocityUnitStr, "mph");
        strcpy(distanceUnitStr, "mi");
        strcpy(odoUnitStr, "mi");
        strcpy(tempUnitStr, "F");
        strcpy(consumptionUnitStr, "Wh/mi"); // Unit for both Trip and Avg Eff
        strcpy(energyUnitStr, "Wh");
        strcpy(capacityUnitStr, "Ah");
    } else { // Metric Units (Kilometers)
        displayVelocity = velocity;
        displayDistance = distance;
        displayOdometer = odometer;
        odoForCalc = displayOdometer; // Use km for calculation
        distForCalc = displayDistance; // Use km for calculation
        displayWhUsed = watthour; // Assign CURRENT TRIP metric net Wh
        displayAhUsed = amphour; // Assign CURRENT TRIP metric net Ah
        displayTempFet = temp_fet; displayTempMotor = temp_motor;
        displayMaxFetTemp = maxfetemp; displayMaxMotorTemp = maxmotortemp;
        displayBatPercentage = batpercentage;
        displayCellVoltage = (CELL_COUNT > 0) ? voltage / (float)CELL_COUNT : 0.0;
        displayPower = power;

        // Assign unit strings
        strcpy(velocityUnitStr, "km/h");
        strcpy(distanceUnitStr, "km");
        strcpy(odoUnitStr, "km");
        strcpy(tempUnitStr, "C");
        strcpy(consumptionUnitStr, "Wh/km"); // Unit for both Trip and Avg Eff
        strcpy(energyUnitStr, "Wh");
        strcpy(capacityUnitStr, "Ah");
    }

    // Calculate Trip Efficiency (using current trip Wh and distance)
    if (distForCalc > 0.01) {
        displayTripEff = displayWhUsed / distForCalc; // Uses CURRENT TRIP Wh (watthour)
    } else {
        displayTripEff = 0.0;
    }

    // Calculate Average Lifetime Efficiency (using ACCUMULATED lifetime NET Wh and odometer)
    if (odoForCalc > 0.1) { // Require a minimum distance for meaningful avg
         displayAvgEff = lifetimeNetWattHour / odoForCalc; // Use ACCUMULATED lifetimeNetWattHour
    } else {
        displayAvgEff = 0.0;
    }
}


// --- Drawing Functions ---
// Now read from 'display*' variables

// Helper function to draw a horizontal dotted line
void drawDottedLine(int x0, int y, int x1, uint16_t color, int spacing) {
    if (x0 > x1) { int temp = x0; x0 = x1; x1 = temp; }
    for (int x = x0; x <= x1; x += spacing) {
        lcd.drawPixel(x, y, color);
    }
}

// --- Screen 1: Main Telemetry Display ---
// Uses F() macro for labels/units
// *** RESTORED Regen Color Logic ***
void drawScreen1() {
  int textHeight = 16; // Size 2 height
  lcd.setTextSize(2);

  // --- Battery indicator outline (Always redraw) ---
  lcd.drawRect(180, 2, 50, 25, WHITE);
  lcd.fillRect(175, 10, 5, 10, WHITE);

  // --- Trip Distance (Always redraw) ---
  lcd.setTextColor(DEFAULT_VALUE_COLOR, BLACK);
  lcd.setCursor(10, 10); lcd.print(displayDistance, 2);
  lcd.setTextColor(DEFAULT_UNIT_COLOR);
  lcd.print(F(" ")); lcd.print(distanceUnitStr); lcd.print(F("   ")); // Use F() for padding

  // --- Battery Percentage (Always redraw) ---
  lcd.setTextColor(DEFAULT_VALUE_COLOR, BLACK);
  // *** FIX: Shifted battery % value slightly left ***
  lcd.setCursor(184, 8); // Was 186
  if (displayBatPercentage < 100) lcd.print(F(" ")); // Use F()
  if (displayBatPercentage < 10) lcd.print(F(" ")); // Use F()
  lcd.print(displayBatPercentage, 0);
  lcd.setTextColor(DEFAULT_UNIT_COLOR);
  lcd.print(F("%")); // Use F()

  // --- Conditional Speed Display ---
  // Uses displayVelocity, compares against last drawn value
  int currentSpeedInt = round(displayVelocity * 10);
  int lastSpeedInt = round(lastDispVelocity1 * 10);
  if (currentSpeedInt != lastSpeedInt || lastDispVelocity1 == UNINITIALIZED_FLOAT) {
      // *** FIX: Increased height of clear rectangle for speed ***
      lcd.fillRect(0, velocity_y, SCR_WD, velocity_height, BLACK); // Clear area first (height increased in constants)
      lcd.setCursor(10, velocity_y);
      lcd.setTextSize(9);
      lcd.setTextColor(DEFAULT_VALUE_COLOR, BLACK);
      if (displayVelocity < 10.0 && displayVelocity >= 0) lcd.print(F(" ")); // Use F()
      lcd.print(displayVelocity, 1);
      lastDispVelocity1 = displayVelocity; // Update last drawn value
  }
  lcd.setTextSize(2); // Reset text size

  // --- Data Rows Below Velocity ---
  int currentY = velocity_y + velocity_height + 10; // Y pos starts below the (now taller) velocity area
  int rowHeight = textHeight + rowSeparatorMargin * 2;
  int separatorY;

  // Helper lambda for drawing rows (uses background clearing)
  // Pass unit string as char* from updateDisplayValues
  auto drawDataRow = [&](const __FlashStringHelper* label, float value, int precision, const char* unit,
                         bool isInt, bool checkNegative)
  {
      uint16_t currentLabelColor = DEFAULT_LABEL_COLOR;
      uint16_t currentUnitColor = DEFAULT_UNIT_COLOR;
      // Check for negative value only if requested, using raw battery current
      if (checkNegative && current < 0.0) {
          currentLabelColor = REGEN_COLOR;
          currentUnitColor = REGEN_COLOR;
      }

      // Draw separator (always draw for simplicity)
      separatorY = currentY - rowSeparatorMargin;
      // *** FIX: Ensure separator Y is below the velocity area ***
      if (separatorY > (velocity_y + velocity_height)) {
          drawDottedLine(separatorStartX, separatorY, separatorEndX, SEPARATOR_DOT_COLOR, separatorDotSpacing);
      }

      // Draw Label (with background clearing)
      lcd.setTextColor(currentLabelColor, BLACK);
      lcd.setCursor(labelStartX1, currentY); lcd.print(label); // Label is already F() string

      // Draw Value (with background clearing)
      lcd.setTextColor(DEFAULT_VALUE_COLOR, BLACK); // Value is always default color
      lcd.setCursor(valueStartX1, currentY);
      if (isInt) lcd.print((int)round(value)); else lcd.print(value, precision);
      lcd.print(F("   ")); // Padding with F()

      // Draw Unit (with background clearing)
      lcd.setTextColor(currentUnitColor, BLACK); // Unit color matches label color
      lcd.setCursor(unitStartX1, currentY); lcd.print(unit); lcd.print(F("   ")); // Padding with F()

      currentY += rowHeight; // Increment Y position
  };

  // Draw the rows using display variables
  drawDataRow(F("Power"), displayPower, 0, " W", true, true); // Check for regen color
  if (CELL_COUNT > 0) {
      drawDataRow(F("Cell V"), displayCellVoltage, 2, " V", false, false); // No regen check needed
  } else {
      lcd.fillRect(0, currentY, SCR_WD, rowHeight, BLACK); // Clear row if skipping
      currentY += rowHeight;
  }
  drawDataRow(F("FET Temp"), displayTempFet, 0, tempUnitStr, true, false); // No regen check needed
  drawDataRow(F("Mot Temp"), displayTempMotor, 0, tempUnitStr, true, false); // No regen check needed
}


// --- Screen 2: Motor Data Display --- MODIFIED for Size 3 Font
void drawScreen2() {
    // Title and Separator
    int titleY = 10;
    int titleHeight = 16; // Title uses size 2
    int separatorY = titleY + titleHeight + 5;
    lcd.setTextColor(White, BLACK);
    lcd.setTextSize(2);
    lcd.setCursor(80, titleY); // Centered-ish for "MOTOR"
    lcd.print(F("MOTOR")); // Use F()
    lcd.fillRect(0, separatorY, SCR_WD, titleSeparatorHeight, TITLE_SEPARATOR_COLOR);

    // Data Rows (Size 3 Font)
    int y = separatorY + titleSeparatorHeight + 15; // Start Y below separator, add more padding
    const int textSize = 3; // <<<< CHANGED TEXT SIZE
    const int textHeight = 24; // Approximate height for size 3 text
    // Adjust row height for larger text and adequate spacing
    const int rowHeight = textHeight + (rowSeparatorMargin * 2) + 4; // Added extra padding

    // Helper lambda for drawing lines (handles regen color, uses BG clearing)
    // Pass unit string as char* from updateDisplayValues
    auto printDataLine = [&](const __FlashStringHelper* label, float value, int precision, const char* unit,
                             bool isInt, bool checkNegative)
    {
        uint16_t currentLabelColor = DEFAULT_LABEL_COLOR;
        uint16_t currentUnitColor = DEFAULT_UNIT_COLOR;

        // Determine color based on negative battery current (regen indicator)
        if (checkNegative && current < 0.0) {
             currentLabelColor = REGEN_COLOR;
             currentUnitColor = REGEN_COLOR;
        }

        // Draw separator
        int sepY_draw = y - rowSeparatorMargin - 2; // Adjust separator position for larger font
        if (sepY_draw > separatorY + titleSeparatorHeight) { // Only draw below title separator
            drawDottedLine(separatorStartX, sepY_draw, separatorEndX, SEPARATOR_DOT_COLOR, separatorDotSpacing);
        }
        lcd.setTextSize(textSize); // Set text size to 3

        // Label (with background clearing) - Use abbreviated labels
        lcd.setTextColor(currentLabelColor, BLACK);
        lcd.setCursor(labelStartX2, y); lcd.print(label); lcd.print(F(" ")); // Use F() for padding

        // Value (with background clearing)
        lcd.setTextColor(DEFAULT_VALUE_COLOR, BLACK);
        lcd.setCursor(valueStartX2, y);
        if (isInt) lcd.print((int)round(value)); else lcd.print(value, precision);
        lcd.print(F(" ")); // Use F() for padding

        // Unit (with background clearing)
        lcd.setTextColor(currentUnitColor, BLACK); // Unit color matches label
        lcd.setCursor(unitStartX2, y); lcd.print(unit); lcd.print(F(" ")); // Use F() for padding

        y += rowHeight; // Increment Y position
    };

    // --- Draw items using Size 3 font and abbreviated labels ---
    // Pass 'true' for checkNegative for items that should change color during regen
    printDataLine(F("Pwr"), displayPower, 0, "W", true, true); // Abbreviated Label
    printDataLine(F("Bat A"), current, 1, "A", false, true); // Abbreviated Label
    printDataLine(F("Phs A"), phase_current, 1, "A", false, true); // Abbreviated Label
    printDataLine(F("RPM"), rpm, 0, "", true, false); // Label already short

    // Clear remaining screen area below the last drawn item
    lcd.fillRect(0, y - rowSeparatorMargin - 2, SCR_WD, SCR_HT - (y - rowSeparatorMargin - 2), BLACK);
}


// --- Screen 3: Temperature Focus Screen ---
// Simplified: Redraws all elements each time (large font = potential flicker)
// Uses F() macro for labels/units
// Reads from 'display*' variables
void drawScreen3() {
  // Title and Separator
  int titleY = 10;
  int titleHeight = 16; // Title uses size 2
  int separatorY = titleY + titleHeight + 5;
  lcd.setTextColor(White, BLACK);
  lcd.setTextSize(2);
  int titleX = 35;
  lcd.setCursor(titleX, titleY);
  if (currentUnits == 1) { lcd.print(F("TEMPERATURE (F)")); } // Use F()
  else { lcd.print(F("TEMPERATURE (C)")); } // Use F()
  lcd.fillRect(0, separatorY, SCR_WD, titleSeparatorHeight, TITLE_SEPARATOR_COLOR);

  // Temp Data (Large Font)
  lcd.setTextSize(3); // Set text size to 3
  int currentY = separatorY + titleSeparatorHeight + 15; // Start Y below separator, add more padding
  const int textHeight = 24; // Approximate height for size 3 text
  // Adjust row height for larger text and adequate spacing
  const int rowHeight = textHeight + (rowSeparatorMargin * 2) + 4; // Added extra padding

  // Helper lambda for temp rows (using BG clearing for simplicity here)
  auto drawTempRow = [&](const __FlashStringHelper* label, float value) {
      int sepY_draw = currentY - rowSeparatorMargin - 2; // Adjust separator position
      if (sepY_draw > separatorY + titleSeparatorHeight) {
          drawDottedLine(separatorStartX, sepY_draw, separatorEndX, SEPARATOR_DOT_COLOR, separatorDotSpacing);
      }
      // Using text with background clearing for Screen 3 for now
      lcd.setTextColor(DEFAULT_LABEL_COLOR, BLACK);
      lcd.setCursor(labelStartX3, currentY); lcd.print(label); lcd.print(F(" ")); // Use F() for padding
      lcd.setTextColor(DEFAULT_VALUE_COLOR, BLACK);
      lcd.setCursor(valueStartX3, currentY); lcd.print((int)round(value)); lcd.print(F("  ")); // Use F() for padding
      currentY += rowHeight;
  };

  // Draw the rows using display variables
  drawTempRow(F("Mot"), displayTempMotor); // Use F()
  drawTempRow(F("Mot Max"), displayMaxMotorTemp); // Use F()
  drawTempRow(F("FET"), displayTempFet); // Use F()
  drawTempRow(F("FET Max"), displayMaxFetTemp); // Use F()

  // Clear remaining screen area below the last drawn item
  lcd.fillRect(0, currentY - rowSeparatorMargin - 2, SCR_WD, SCR_HT - (currentY - rowSeparatorMargin - 2), BLACK);
}


// --- Screen 4: Trip Information Screen --- MODIFIED
// Simplified: Redraws all elements each time (using BG clearing)
// Uses F() macro for labels/units
// Added Avg Eff, Renamed Trip Eff, Added Ah Used, Reordered
// Reads from 'display*' variables
// *** ADDED Regen Color Logic ***
void drawScreen4() {
    // Title and Separator
    int titleY = 10;
    int titleHeight = 16;
    int separatorY = titleY + titleHeight + 5;
    lcd.setTextColor(White, BLACK);
    lcd.setTextSize(2);
    lcd.setCursor(60, titleY); lcd.print(F("TRIP INFO")); // Use F()
    lcd.fillRect(0, separatorY, SCR_WD, titleSeparatorHeight, TITLE_SEPARATOR_COLOR);

    // Trip Data
    lcd.setTextSize(2);
    int currentY = separatorY + titleSeparatorHeight + 10;
    int labelX = 15;
    int valueX = 125;
    int unitX = 175;
    const int textHeight = 16;
    const int rowSpacing = 8;
    const int rowHeight = textHeight + rowSpacing;

    // Helper lambda for trip data lines (using BG clearing)
    // Pass unit string as char* from updateDisplayValues
    // *** ADDED checkNegative parameter ***
    auto drawTripLine = [&](const __FlashStringHelper* label, float value, int precision, const char* unit, bool checkNegative) {
        int sepY_draw = currentY - (rowSpacing / 2) - 1 ;
        if (currentY > separatorY + titleSeparatorHeight + 5) {
             drawDottedLine(separatorStartX, sepY_draw, separatorEndX, SEPARATOR_DOT_COLOR, separatorDotSpacing);
        }

        uint16_t currentLabelColor = DEFAULT_LABEL_COLOR;
        uint16_t currentUnitColor = DEFAULT_UNIT_COLOR;
        // Check for regen color only if requested, using raw battery current 'current'
        if (checkNegative && current < 0.0) {
             currentLabelColor = REGEN_COLOR;
             currentUnitColor = REGEN_COLOR;
        }

        lcd.setTextColor(currentLabelColor, BLACK); // Use determined color
        lcd.setCursor(labelX, currentY); lcd.print(label); lcd.print(F(" ")); // Use F() for padding
        lcd.setTextColor(DEFAULT_VALUE_COLOR, BLACK); // Value always default color
        lcd.setCursor(valueX, currentY);
        lcd.print(value, precision); lcd.print(F(" ")); // Use F() for padding
        lcd.setTextColor(currentUnitColor, BLACK); // Use determined color
        lcd.setCursor(unitX, currentY); lcd.print(unit); lcd.print(F("  ")); // Use F() for padding
        currentY += rowHeight;
    };

    // Draw the lines - MODIFIED ORDER AND CONTENT using display variables
    // Pass 'true' to checkNegative for Wh Used and Ah Used (based on raw battery current)
    drawTripLine(F("ODO"), displayOdometer, 1, odoUnitStr, false);
    drawTripLine(F("Avg Eff"), displayAvgEff, 1, consumptionUnitStr, false); // Avg Eff doesn't change color
    drawTripLine(F("Dist"), displayDistance, 2, distanceUnitStr, false);
    drawTripLine(F("Trip Eff"), displayTripEff, 1, consumptionUnitStr, false); // Trip Eff doesn't change color
    drawTripLine(F("Wh used"), displayWhUsed, 1, energyUnitStr, true); // Check regen (uses displayWhUsed which is current trip)
    drawTripLine(F("Ah used"), displayAhUsed, 2, capacityUnitStr, true); // Check regen (uses displayAhUsed which is current trip) - NEW

    // Clear remaining screen area below the last drawn item
    lcd.fillRect(0, currentY - (rowSpacing / 2) -1, SCR_WD, SCR_HT - (currentY - (rowSpacing / 2) -1), BLACK);
}


// --- Screen 5: Settings Screen ---
// Settings screen redraws fully on focus change anyway
// Uses F() macro for labels/units
// "CLEAR ODO" now clears ODO and Avg Eff stats
void drawSettingsScreen() {
    lcd.setTextSize(2);
    int currentY = 10;
    int titleHeight = 16;
    int itemHeight = 16;
    int itemSpacing = 10;
    int lineY;
    int textW; // Calculated text width

    // Title
    lcd.setTextColor(White, BLACK);
    lcd.setCursor(70, currentY);
    lcd.print(F("SETTINGS")); // Use F()
    currentY += titleHeight + 5;
    lcd.fillRect(0, currentY, SCR_WD, titleSeparatorHeight, TITLE_SEPARATOR_COLOR); // Title separator
    currentY += titleSeparatorHeight + 15;

    // Units Label
    lcd.setTextColor(DEFAULT_LABEL_COLOR, BLACK);
    lcd.setCursor(settingsLabelX, currentY);
    lcd.print(F("Units")); // Use F()
    currentY += itemHeight + itemSpacing;

    // Units Options (Standard / Metric)
    int unitsOptionX = settingsUnitsOptionX;

    // Standard Option
    const char* standardStr = "Standard"; // Keep as char* for strlen
    textW = strlen(standardStr) * 6 * 2; // Approx width
    lcd.setCursor(unitsOptionX, currentY);
    if (settingsFocus == 0) { // Focused
        lcd.fillRect(unitsOptionX - 2, currentY - 1, textW + 4, itemHeight + 2, SETTINGS_FOCUS_BG_COLOR); // Draw BG
        lcd.setTextColor(SETTINGS_FOCUS_COLOR); // Set text color
    } else { // Not focused
        lcd.fillRect(unitsOptionX - 2, currentY - 1, textW + 4, itemHeight + 2, BLACK); // Clear BG
        lcd.setTextColor(SETTINGS_ITEM_COLOR); // Set text color
    }
    lcd.print(F("Standard")); // Use F() for drawing
    lineY = currentY + itemHeight + 1; // Underline position
    if (currentUnits == 1) { lcd.fillRect(unitsOptionX, lineY, textW, 1, SETTINGS_SELECTED_COLOR); } // Draw underline if selected
    else { lcd.fillRect(unitsOptionX, lineY, textW, 1, BLACK); } // Clear underline if not selected
    unitsOptionX += settingsUnitsOptionSpacing; // Move X for next option

    // Metric Option
    const char* metricStr = "Metric"; // Keep as char* for strlen
    textW = strlen(metricStr) * 6 * 2;
    lcd.setCursor(unitsOptionX, currentY);
    if (settingsFocus == 1) { // Focused
        lcd.fillRect(unitsOptionX - 2, currentY - 1, textW + 4, itemHeight + 2, SETTINGS_FOCUS_BG_COLOR);
        lcd.setTextColor(SETTINGS_FOCUS_COLOR);
    } else { // Not focused
        lcd.fillRect(unitsOptionX - 2, currentY - 1, textW + 4, itemHeight + 2, BLACK);
        lcd.setTextColor(SETTINGS_ITEM_COLOR);
    }
    lcd.print(F("Metric")); // Use F() for drawing
    lineY = currentY + itemHeight + 1;
    if (currentUnits == 0) { lcd.fillRect(unitsOptionX, lineY, textW, 1, SETTINGS_SELECTED_COLOR); }
    else { lcd.fillRect(unitsOptionX, lineY, textW, 1, BLACK); }
    currentY += itemHeight + itemSpacing + 5; // Move Y down

    // Screen Rotation Label
    lcd.setTextColor(DEFAULT_LABEL_COLOR, BLACK);
    lcd.setCursor(settingsLabelX, currentY);
    lcd.print(F("Screen Rotation")); // Use F()
    currentY += itemHeight + itemSpacing;

    // Rotation Options (0, 90, 180, 270)
    int rotOptionsX = settingsRotOptionX;
    const char* rotLabels[] = {"0", "90", "180", "270"}; // Keep as char* for strlen
    const __FlashStringHelper* rotLabelsF[] = {F("0"), F("90"), F("180"), F("270")}; // Use F() for drawing
    for (int i = 0; i < 4; i++) {
        int labelLen = strlen(rotLabels[i]);
        textW = labelLen * 6 * 2; // Approx width
        lcd.setCursor(rotOptionsX, currentY);
        if (settingsFocus == (i + 2)) { // Focused
             lcd.fillRect(rotOptionsX - 2, currentY - 1, textW + 4, itemHeight + 2, SETTINGS_FOCUS_BG_COLOR);
             lcd.setTextColor(SETTINGS_FOCUS_COLOR);
        } else { // Not focused
             lcd.fillRect(rotOptionsX - 2, currentY - 1, textW + 4, itemHeight + 2, BLACK);
             lcd.setTextColor(SETTINGS_ITEM_COLOR);
        }
        lcd.print(rotLabelsF[i]); // Draw text using F() version
        lineY = currentY + itemHeight + 1; // Underline position
        if (currentRotation == i) { lcd.fillRect(rotOptionsX, lineY, textW, 1, SETTINGS_SELECTED_COLOR); } // Draw underline if selected
        else { lcd.fillRect(rotOptionsX, lineY, textW, 1, BLACK); } // Clear underline
        rotOptionsX += settingsRotOptionSpacing; // Move X for next option
    }
    currentY += itemHeight + itemSpacing + 5; // Move Y down

    // Clear Odometer Option (Label remains "CLEAR ODO", but action clears both)
    const char* clearOdoStr = "CLEAR ODO"; // Keep as char* for strlen
    textW = strlen(clearOdoStr) * 6 * 2;
    lcd.setCursor(settingsLabelX, currentY);
    if (settingsFocus == CLEAR_ODO_FOCUS_INDEX) { // Focused
        lcd.fillRect(settingsLabelX - 2, currentY - 1, textW + 4, itemHeight + 2, SETTINGS_FOCUS_BG_COLOR);
        lcd.setTextColor(SETTINGS_FOCUS_COLOR);
    } else { // Not focused
        lcd.fillRect(settingsLabelX - 2, currentY - 1, textW + 4, itemHeight + 2, BLACK);
        lcd.setTextColor(DEFAULT_LABEL_COLOR); // Use DEFAULT label color when not focused
    }
    lcd.print(F("CLEAR ODO")); // Use F() for drawing
    currentY += itemHeight + itemSpacing + 10; // Move Y down

    // EXIT Option (Large Font)
    lcd.setTextSize(3);
    const char* exitStr = "EXIT"; // Keep as char* for strlen
    int exitTextW = strlen(exitStr) * 6 * 3; // Approx width for size 3
    int exitX = (SCR_WD - exitTextW) / 2; // Center horizontally
    int exitY = SCR_HT - 30; // Position near bottom
    int exitItemH = 24; // Height for size 3
    lcd.setCursor(exitX, exitY);
    if (settingsFocus == EXIT_FOCUS_INDEX) { // Focused
        lcd.fillRect(exitX - 2, exitY - 1, exitTextW + 4, exitItemH + 2, SETTINGS_FOCUS_BG_COLOR);
        lcd.setTextColor(SETTINGS_FOCUS_COLOR);
    } else { // Not focused
        lcd.fillRect(exitX - 2, exitY - 1, exitTextW + 4, exitItemH + 2, BLACK);
        lcd.setTextColor(DEFAULT_LABEL_COLOR); // Use DEFAULT label color when not focused
    }
    lcd.print(F("EXIT")); // Use F() for drawing
}

// --- END OF FILE ---
